# -*- coding: utf-8 -*-

from .data_parser import JsonParser
from .consumer import Consumer


class DebugConsumer(Consumer):
    def __init__(self, product_id, data_source_id, server_host, data_parser=None,
                 retry_limit=3, request_timeout=2, retry_backoff_factor=0.25, verify_cert=True):
        super(DebugConsumer, self).__init__(product_id, data_source_id, server_host,
                                            retry_limit, request_timeout, retry_backoff_factor, verify_cert)
        self._data_parse = data_parser or JsonParser()
        print("GrowingIO Configuration:{"
              + "productId=" + product_id + ","
              + "dataSourceId=" + data_source_id + ","
              + "serverHost=" + server_host + "}"
              )

    def send(self, message):
        print('SEND MESSAGE:' + str(message))
        if hasattr(message, 'event_type'):
            request_url = self.endpoints['collect']
        else:
            request_url = self.endpoints['item']

        messages = [message]
        self.post_data(request_url, messages)

    def post_data(self, request_url, messages):
        stm = self._data_parse.get_timestamp()
        data = self._data_parse.get_bytes(messages, stm)
        print("╔═════════════════════════════HTTP POST══════════════════════════════════")
        print("--> POST " + request_url + " HTTP_1_1")
        print(' (' + str(len(data)) + '-byte body)')
        print(self._data_parse.get_headers())
        print(data)

        result = self.send_data(
            request_url,
            data,
            params=self._data_parse.get_params(stm=stm),
            headers=self._data_parse.get_headers()
        )
        print('<-- RESULT:' + str(result))
        print("╚═════════════════════════════POST  END══════════════════════════════════")



# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals
import snappy
import six
from growingio_tracker.data_parser import JsonParser


class SnappyParser(JsonParser):

    def __init__(self, crypt_data=False):
        super(SnappyParser, self).__init__()
        self._crypt_data = crypt_data

    def get_bytes(self, messages, stm=None):
        batch = []
        for message in messages:
            batch.append(self.json(message))
        batch_json = '[{0}]'.format(','.join(batch))
        compress_data = snappy.compress(batch_json)
        if self._crypt_data:
            return self._xor_crypt(compress_data, stm)
        return compress_data

    def _xor_crypt(self, raw_data, password):
        if six.PY3:
            xor = bytearray()
            for nowByte in bytearray(raw_data):
                newByte = nowByte ^ int(password & 0xff)
                xor.extend(bytes([newByte]))
            return xor
        # elif six.PY2:
        #     xor = ''.join(chr(ord(x) ^ (password & 0xff)) for (x) in raw_data)
        #     return xor
        else:
            self._crypt_data = False
            return raw_data

    def get_headers(self):
        if self._crypt_data and six.PY3:
            return {'content-type': 'application/json', 'X-Compress-Codec': '2', 'X-Crypt-Codec': '1'}
        return {'content-type': 'application/json', 'X-Compress-Codec': '2'}



# -*- coding: utf-8 -*-

from __future__ import absolute_import
import threading
from .default_consumer import DefaultConsumer


class FlushThread(threading.Thread):
    '''
    FlushThread is used to asynchronously flush the events stored in
    the AsyncBufferedConsumer buffers.
    '''

    def __init__(self, consumer):
        threading.Thread.__init__(self)
        self._consumer = consumer

        self._stop_event = threading.Event()
        self._finished_event = threading.Event()

    def stop(self):
        self._stop_event.set()
        self._finished_event.wait()

    def run(self):
        while True:
            self._consumer.need_flush.wait(self._consumer.flush_after)
            if self._consumer.flush():
                self._consumer.need_flush.clear()
            if self._stop_event.isSet():
                break
        self._finished_event.set()


class AsyncBufferedConsumer(DefaultConsumer):

    def __init__(self, product_id, data_source_id, server_host, data_parser=None,
                 flush_after=10, max_size=500, request_timeout=2,
                 retry_limit=3, retry_backoff_factor=0.25, verify_cert=True):

        super(AsyncBufferedConsumer, self).__init__(
            product_id, data_source_id, server_host, data_parser,
            request_timeout=request_timeout,
            retry_limit=retry_limit,
            retry_backoff_factor=retry_backoff_factor,
            verify_cert=verify_cert,
        )

        # remove the minimum max size that the SynchronousBufferedConsumer
        self._max_size = max_size
        self.flush_after = flush_after

        self.need_flush = threading.Event()

        self._async_events = []
        self._async_items = []

        self._async_buffers = {
            'events': [],
            'items': [],
        }

        self._flushing_thread = FlushThread(self)
        self._flushing_thread.daemon = True
        self._flushing_thread.start()

        self.flush_lock = threading.Lock()
        self.flushing_thread = None

    def _flush_thread_is_free(self):
        return self.flushing_thread is None or not self.flushing_thread.is_alive()

    def _should_flush(self):
        full = len(self._async_events) >= self._max_size or len(self._async_items) > 0

        if full:
            return True

        return False

    def send(self, message):

        if hasattr(message, 'event_type'):
            buf = self._async_events
            buf.append(message)
        else:
            buf = self._async_items
            buf.append(message)

        should_flush = self._should_flush()
        if should_flush:
            self.need_flush.set()
            # self.flush()

    def stop(self):
        self._flushing_thread.stop()

    def flush(self):
        with self.flush_lock:
            if self._flush_thread_is_free():
                self.transfer_buffers()
                self._sync_flush()
                flushing = True
            else:
                flushing = False

        return flushing

    def transfer_buffers(self):
        buf = self._async_events
        while buf:
            self._async_buffers['events'].append(buf.pop(0))
        buf = self._async_items
        while buf:
            self._async_buffers['items'].append(buf.pop(0))

    def _sync_flush(self):
        if len(self._async_buffers['events']) > 0:
            request_url = self.endpoints['collect']
            buf = self._async_buffers['events']
            stm = self._data_parse.get_timestamp()
            while buf:
                batch = buf[:self._max_size]
                self.post_data(request_url, batch)
                buf = buf[self._max_size:]
            self._async_buffers['events'] = buf

        if len(self._async_buffers['items']) > 0:
            request_url = self.endpoints['item']
            buf = self._async_buffers['items']
            while buf:
                batch = buf[:self._max_size]
                self.post_data(request_url, batch)
                buf = buf[self._max_size:]
            self._async_buffers['items'] = buf



from codecs import open
from os import path
import re
from setuptools import setup, find_packages


def read(*paths):
    filename = path.join(path.abspath(path.dirname(__file__)), *paths)
    with open(filename, encoding='utf-8') as f:
        return f.read()


def find_version(*paths):
    contents = read(*paths)
    match = re.search(r'^VERSION = [\'"]([^\'"]+)[\'"]', contents, re.M)
    if not match:
        raise RuntimeError('Unable to find version string.')
    return match.group(1)

# 上传到 pypi
# python3 setup.py sdist bdist_wheel
# twine upload dist/*
# 账号信息在飞书群的公告里
setup(
    name='growingio_tracker',
    version=find_version('growingio_tracker', 'version.py'),
    description='GrowingIO Tracker library for Python',
    long_description=read('README.rst'),
    url='https://github.com/growingio/growingio-python-sdk-v3/',
    author='cpacm,GrowingIO SDK Team',
    author_email='sdk-integration@growingio.com',
    license='Apache',
    python_requires='>=2.7, !=3.4.*',
    install_requires=[
        'six>=1.9.0',
        'requests>=2.4.2',
        'urllib3',
    ],
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    keywords='growingio python analytics',
    packages=find_packages(),
)
